@@ -18,23 +18,27 @@ pomp(
      times="day",
      t0=14,
      rprocess=discrete.time.sim(
-       step.fun="_blowfly_model_simulator",
+       step.fun="_blowfly_simulator_one",
        delta.t=1,
+       PACKAGE="pomp"
        ),
-     paramnames=c("log.P","log.N0","log.delta","log.sigma.P","log.sigma.d","tau","log.sigma.y"),
+     rmeasure="_blowfly_rmeasure",
+     dmeasure="_blowfly_dmeasure",
+     PACKAGE="pomp",
+     paramnames=c("P","N0","delta","sigma.P","sigma.d","sigma.y"),
      statenames=c("N1","R","S","e","eps"),
      obsnames=c("y"),
-     measurement.model=y~nbinom(mu=N1,size=exp(-2*log.sigma.y)),
      y.init=with( ## initial data
        raw.data,
-       approx(
-              x=day,
-              y=y,
-              xout=seq(from=0,to=14,by=1),
-              rule=2
-              )$y
+       approx(x=day,y=y,xout=seq(from=0,to=14,by=1),rule=2)$y
        ),
 #     y.init=c(948, 948, 942, 930, 911, 885, 858, 833.7, 801, 748.3, 676, 589.8, 504, 434.9, 397),
+     parameter.inv.transform=function(params,...) {
+       log(params)
+     },
+     parameter.transform=function(params,...) {
+       exp(params)
+     },
      initializer=function (params, t0, y.init, ...) {
        ntau <- length(y.init)
        n <- y.init[ntau:1]
@@ -44,25 +48,15 @@ pomp(
      ) -> blowflies1
 
 pomp(
-     data=subset(raw.data[c("day","y")],day>14&day<400),
-     times="day",
-     t0=14,
+     blowflies1,
      rprocess=discrete.time.sim(
-       step.fun="_blowfly_model_simulator",
+       step.fun="_blowfly_simulator_two",
        delta.t=2,
+       PACKAGE="pomp"
        ),
-     paramnames=c("log.P","log.N0","log.delta","log.sigma.P","log.sigma.d","tau","log.sigma.y"),
-     statenames=c("N1","R","S","e","eps"),
-     obsnames=c("y"),
-     measurement.model=y~nbinom(mu=N1,size=exp(-2*log.sigma.y)),
      y.init=with( ## initial data
        raw.data,
-       approx(
-              x=day,
-              y=y,
-              xout=seq(from=0,to=14,by=2),
-              rule=2
-              )$y
+       approx(x=day,y=y,xout=seq(from=0,to=14,by=2),rule=2)$y
        ),
      #y.init=c(948, 942, 911, 858, 801, 676, 504, 397),
      initializer=function (params, t0, y.init, ...) {
@@ -74,26 +68,24 @@ pomp(
      ) -> blowflies2
 
 ## mle from search to date
-coef(blowflies1) <- c(
-                    log.P = 1.189 , 
-                    log.delta = -1.828 , 
-                    log.N0 = 6.522 , 
-                    log.sigma.P = 0.301 , 
-                    log.sigma.d = -0.292 , 
-                    log.sigma.y = -3.625 , 
-                    tau = 14 
-                    )
+coef(blowflies1,transform=TRUE) <- c(
+                  P = 1.189, 
+                  delta = -1.828, 
+                  N0 = 6.522, 
+                  sigma.P = 0.301, 
+                  sigma.d = -0.292, 
+                  sigma.y = -3.625
+                  )
 
 ## mle from search to date
-coef(blowflies2) <- c(
-                    log.P = 1.005 , 
-                    log.delta = -1.75 , 
-                    log.N0 = 6.685 , 
-                    log.sigma.P = 0.366 , 
-                    log.sigma.d = -0.274 , 
-                    log.sigma.y = -4.524 , 
-                    tau = 7 
-                    )
+coef(blowflies2,transform=TRUE) <- c(
+                  P = 1.005, 
+                  delta = -1.75, 
+                  N0 = 6.685, 
+                  sigma.P = 0.366, 
+                  sigma.d = -0.274, 
+                  sigma.y = -4.524
+                  )
 
 test <- FALSE
 if(test){
@@ -109,9 +101,9 @@ if(test){
 
   ## check that it matches the deterministic skeleton when noise is small
   params.1.skel <- coef(blowflies1)
-  params.1.skel["log.sigma.P"] <- log(0.00001)
-  params.1.skel["log.sigma.d"] <- log(0.00001)
-  params.1.skel["log.sigma.y"] <- log(0.00001)
+  params.1.skel["sigma.P"] <- 0.00001
+  params.1.skel["sigma.d"] <- 0.00001
+  params.1.skel["sigma.y"] <- 0.00001
   simulate(blowflies1,params=params.1.skel,nsim=1,seed=73691676L) -> b1.skel
   plot(obs(blowflies1)['y',],ty='l',lty="dashed")
   lines(obs(b1.skel)['y',],ty='l')
