@@ -36,4 +36,4 @@ simulate(
 ou2 <- window(ou2,end=100)
 timezero(ou2) <- 0
 
-save(ou2,file="ou2.rda")
+save(ou2,file="ou2.rda",compress="xz")
